--------------------------------------------------------------------------------
-- Copyright (c) 1995-2003 Xilinx, Inc.
-- All Right Reserved.
--------------------------------------------------------------------------------
--   ____  ____ 
--  /   /\/   / 
-- /___/  \  /    Vendor: Xilinx 
-- \   \   \/     Version : 7.1.03i
--  \   \         Application : ISE WebPACK
--  /   /         Filename : tb.vhw
-- /___/   /\     Timestamp : Sat Oct 22 20:19:18 2005
-- \   \  /  \ 
--  \___\/\___\ 
--
--Command: 
--Design Name: tb
--Device: Xilinx
--

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
USE IEEE.STD_LOGIC_TEXTIO.ALL;
USE STD.TEXTIO.ALL;

ENTITY tb IS
END tb;

ARCHITECTURE testbench_arch OF tb IS
    FILE RESULTS: TEXT OPEN WRITE_MODE IS "results.txt";

    COMPONENT dffd
        PORT (
            ISPTX : InOut std_logic;
            ISPRX : InOut std_logic;
            AVRTXD0 : InOut std_logic;
            AVRRXD0 : InOut std_logic;
            AVRTXD1 : InOut std_logic;
            AVRRXD1 : InOut std_logic;
            M32TXD : InOut std_logic;
            M32RXD : InOut std_logic;
            UARTTX : InOut std_logic;
            UARTRX : InOut std_logic;
            USELECT : In std_logic;
            clk : In std_logic;
            AVRRST : In std_logic
        );
    END COMPONENT;

    SIGNAL ISPTX : std_logic := '0';
    SIGNAL ISPRX : std_logic := '0';
    SIGNAL AVRTXD0 : std_logic := '0';
    SIGNAL AVRRXD0 : std_logic := '0';
    SIGNAL AVRTXD1 : std_logic := '0';
    SIGNAL AVRRXD1 : std_logic := '0';
    SIGNAL M32TXD : std_logic := '0';
    SIGNAL M32RXD : std_logic := '0';
    SIGNAL UARTTX : std_logic := '0';
    SIGNAL UARTRX : std_logic := '0';
    SIGNAL USELECT : std_logic := '0';
    SIGNAL clk : std_logic := '0';
    SIGNAL AVRRST : std_logic := '0';

    SHARED VARIABLE TX_ERROR : INTEGER := 0;
    SHARED VARIABLE TX_OUT : LINE;

    constant PERIOD : time := 200 ns;
    constant DUTY_CYCLE : real := 0.5;
    constant OFFSET : time := 0 ns;

    BEGIN
        UUT : dffd
        PORT MAP (
            ISPTX => ISPTX,
            ISPRX => ISPRX,
            AVRTXD0 => AVRTXD0,
            AVRRXD0 => AVRRXD0,
            AVRTXD1 => AVRTXD1,
            AVRRXD1 => AVRRXD1,
            M32TXD => M32TXD,
            M32RXD => M32RXD,
            UARTTX => UARTTX,
            UARTRX => UARTRX,
            USELECT => USELECT,
            clk => clk,
            AVRRST => AVRRST
        );

        PROCESS    -- clock process for AVRRST
        BEGIN
            WAIT for OFFSET;
            CLOCK_LOOP : LOOP
                AVRRST <= '0';
                WAIT FOR (PERIOD - (PERIOD * DUTY_CYCLE));
                AVRRST <= '1';
                WAIT FOR (PERIOD * DUTY_CYCLE);
            END LOOP CLOCK_LOOP;
        END PROCESS;

        PROCESS
            BEGIN
                -- -------------  Current Time:  85ns
                WAIT FOR 85 ns;
                USELECT <= '1';
                -- -------------------------------------
                -- -------------  Current Time:  285ns
                WAIT FOR 200 ns;
                -- -------------------------------------
                -- -------------  Current Time:  485ns
                WAIT FOR 200 ns;
                USELECT <= '0';
                -- -------------------------------------
                WAIT FOR 715 ns;

                IF (TX_ERROR = 0) THEN
                    STD.TEXTIO.write(TX_OUT, string'("No errors or warnings"));
                    STD.TEXTIO.writeline(RESULTS, TX_OUT);
                    ASSERT (FALSE) REPORT
                      "Simulation successful (not a failure).  No problems detected."
                      SEVERITY FAILURE;
                ELSE
                    STD.TEXTIO.write(TX_OUT, TX_ERROR);
                    STD.TEXTIO.write(TX_OUT,
                        string'(" errors found in simulation"));
                    STD.TEXTIO.writeline(RESULTS, TX_OUT);
                    ASSERT (FALSE) REPORT "Errors found during simulation"
                         SEVERITY FAILURE;
                END IF;
            END PROCESS;

    END testbench_arch;

